/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is gContactSync.
 *
 * The Initial Developer of the Original Code is
 * Josh Geenen <gcontactsync@pirules.net>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
/**
 * oMailList
 * oMailList is an abstraction of a mailing list that facilitates getting the
 * cards contained within the actual list as well as accessing and modifying the
 * list and its properties.
 * @constructor
 * @class
 */
function oMailList(aList, aParentDirectory, aNew) {
  if (!aParentDirectory || !(aParentDirectory instanceof oAddressBook))
    throw "Error - invalid address book supplied to the oMailList Constructor";
  this.mParent = aParentDirectory;
  this.mParent.checkList(aList, "oMailList constructor");
  this.mList = aList;
  this.mList.QueryInterface(Ci.nsIAbMDBDirectory);
  this.mNew = aNew;
  if (!aNew)
    this.getAllCards();
}

oMailList.prototype = {
  mCards: [],
  mCardsUpdate: false,
  /**
   * oMailList.setName
   * Sets the name of this list. The update method must be called in order for
   * the change to become permanent.
   * @param aName The new name for the list.
   */
  setName: function oMailList_setName(aName) {
    this.mList.dirName = aName;
  },
  /**
   * oMailList.getName
   * Returns the name of this list.
   * @return The name of this list.
   */
  getName: function oMailList_getName() {
    return this.mList.dirName;
  },
  /**
   * oMailList.hasCard
   * Returns the card in this mail list, if any, with the same (not-null)
   * value for the GoogleID attribute, or, if the GoogleID is null, if the
   *         display name, primary, and second emails are the same.
   * @param aCard The card being searched for.
   * @return The card in this list, if any, with the same, and non-null value
   *         for its GoogleID attribute, or, if the GoogleID is null, if the
   *         display name, primary, and second emails are the same.
   */
  hasCard: function oMailList_hasCard(aCard) {
    oAbManager.checkCard(aCard);
    // get all of the cards in this list again, if necessary
    if (this.mCardsUpdate || this.mCards.length == 0)
      this.getAllCards();
    for (var i = 0, length = this.mCards.length; i < length; i++) {
      var card = this.mCards[i];
      var aCardID = oAbManager.getCardValue(aCard, "GoogleID");
      // if it is an old card (has id) compare IDs
      if (aCardID) {
        if (aCardID == oAbManager.getCardValue(card, "GoogleID"))
          return card;
      }
      // else check that display name, primary and second email are equal
      else if (oAbManager.getCardValue(aCard, "DisplayName") ==
                                      oAbManager.getCardValue(card,"DisplayName")
              && oAbManager.getCardValue(aCard, "PrimaryEmail") ==
                                        oAbManager.getCardValue(card, "PrimaryEmail")
              && oAbManager.getCardValue(aCard, "SecondEmail") ==
                                        oAbManager.getCardValue(card, "SecondEmail"))
        return card;
    }
  },
  /**
   * oMailList.setNickName
   * Sets the nick name for this mailing list.  The update method must be
   * called in order for the change to become permanent.
   * @param aNickName The new nick name for this mailing list.
   */
  setNickName: function oMailList_setNickName(aNickName) {
    this.mList.listNickName = aNickName;
  },
  /**
   * oMailList.getNickName
   * Returns the nick name of this mailing list.
   * @return The nick name of this mailing list.
   */
  getNickName: function oMailList_getNickName() {
    return this.mList.listNickName;
  },
  /**
   * oMailList.setDescription
   * Sets the description for this mailing list.  The update method must be
   * called in order for the change to become permanent.
   * @param aDescription The new description for this mailing list.
   */
  setDescription: function oMailList_setDescription(aDescription) {
    this.mList.description = aDescription;
  },
  /**
   * oMailList.getDescription
   * Returns the description of this mailing list.
   * @return The description of this mailing list.
   */
  getDescription: function oMailList_getDescription() {
    return this.mList.description;
  },
  /**
   * oMailList.addCard
   * Adds a card to this mailing list without checking if it already exists.
   * @param aCard The card to add to this mailing list.
   * @return A real card (MDB card prior to 413260).
   */
  addCard: function oMailList_addCard(aCard) {
    oAbManager.checkCard(aCard);
    var realCard = this.mList.addCard(aCard);
    this.mCards.push(realCard);
    return realCard;
  },
  /**
   * oMailList.getURI
   * Returns the uniform resource identifier (URI) for this mailing list.
   * @return The URI of this list.
   */
  getURI: function oMailList_getURI() {
    if (this.mList.URI)
      return this.mList.URI;
    return this.mList.getDirUri();
  },
  /**
   * oMailList.getAllCards
   * Returns an array of all of the cards in this mailing list.
   * @return An array containing all of the cards in this mailing list.
   */
  getAllCards: function oMailList_getAllCards() {
    // NOTE: Sometimes hasMoreElements fails if mail lists aren't working
    // properly, but it shouldn't be caught or the sync won't function properly
    this.mCards = [];
    var iter = this.mList.childCards;
    if (oAbManager.mVersion == 3) { // TB 3
      while (iter.hasMoreElements()) {
        data = iter.getNext();
        if (data instanceof nsIAbCard)
          this.mCards.push(data);
      }
    }
    else { // TB 2
      // use nsIEnumerator...
      try {
        iter.first();
        do {
          var data = iter.currentItem();
          if(data instanceof nsIAbCard)
            this.mCards.push(data);
          iter.next();
        } while (Components.lastResult == 0);
      } catch(e) {oLOGGER.VERBOSE_LOG(e);} // error is expected when finished   
    }
    return this.mCards;
  },
  /**
   * oMailList.deleteCards
   * Deletes all of the cards in the array of cards from this list.
   * @param aCards The array of cards to delete from this mailing list.
   */
  deleteCards: function oMailList_deleteCards(aCards) {
    if (!(aCards && aCards.length && aCards.length > 0))
      return;
    var arr;
    if (oAbManager.mVersion == 3) { // TB 3
      arr = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
      for (var i = 0; i < aCards.length; i++) {
        oAbManager.checkCard(aCards[i]);
        arr.appendElement(aCards[i], false);
      }
    }
    else { // TB 2
      arr =  Cc["@mozilla.org/supports-array;1"]
              .createInstance(Ci.nsISupportsArray);
      for (var i = 0; i < aCards.length; i++) {
        oAbManager.checkCard(aCards[i]);
        arr.AppendElement(aCards[i], false);
      }
    }
    try {
      if (arr) { // make sure arr isn't null (mailnews bug 448165)
        this.mCardsUpdate = true; // update mCards when used
        this.mList.deleteCards(arr);
      }
    }
    catch(e) {
      oLOGGER.LOG_WARNING("Error while deleting cards from a mailing list", e);
    }
    this.mCards = this.getAllCards();
  },
  /**
   * oMailList.delete
   * Deletes this mailing list from its parent address book.
   */
  delete: function oMailList_delete() {
    this.mParent.mDirectory.deleteDirectory(this.mList);
    this.mCards = [];
    // make sure the functions don't do anything
    for (var i in this) {
      if (i instanceof Function)
        i = function() {};
    }
  },
  /**
   * oMailList.update
   * Updates this mail list.
   */
  update: function oMailList_update() {
    try {
      if (oAbManager.mVersion == 3)
        this.mList.editMailListToDatabase(null);
      else
        this.mList.editMailListToDatabase(this.getURI(), null);
    }
    catch(e) { oLOGGER.LOG_WARNING("Unable to update mail list", e);}
  },
  /**
   * oMailList.getGroupID
   * Gets and returns the ID of the group in Google with which this Mail List
   * is synchronized, if any.  If not found, returns "no id found" with a space
   * and the current time in microseconds since the epoch.
   * @return The ID of the group with which this directory is synchronized.
   */
   getGroupID: function oMailList_getGroupID() {
     // first see if the nickname is the group id
     var id = this.getNickName();
     if (id.indexOf("http://www.google.com/m8/feeds/groups") == -1)
       id = this.getDescription(); // if it isn't, get the description
     // finally, set it as "no id found" with the current time
     if (id.indexOf("http://www.google.com/m8/feeds/groups") == -1)
       id = "no id found " + (new Date).getTime();
     return id;
   },
   /**
   * oAddressBook.getGroupID
   * Setsthe ID of the group in Google with which this Mail List is
   * synchronized.
   * @return The ID of the group with which this directory is synchronized.
   */
   setGroupID: function oMailList_setGroupID(aGroupID) {
     this.setNickName(aGroupID);
   }
};
